#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, string

assert(os.environ["OBT_SUBSPACE"]=="host")

parser = argparse.ArgumentParser(description='obt.build dep information')
parser.add_argument('dependency', metavar='D', type=str, help='a dependency you want information on')

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

depname = _args["dependency"]

from obt import dep, path
from obt.deco import Deco
deco = Deco()

instance = dep.instance(depname)


def print_item(key,val):
 dstr = deco.inf(depname)
 kstr = deco.key(key)
 vstr = deco.val(val)
 print("%s.%s = %s"%(dstr,kstr,vstr))

src_present = str(instance.areRequiredSourceFilesPresent())
if src_present=="None":
   src_present = "False"

bin_present = str(instance.areRequiredBinaryFilesPresent())
if bin_present=="None":
   bin_present = "False"

decl_deps = list()
for dep_name in instance._required_deps.keys():
  dep_inst = instance._required_deps[dep_name]
  decl_deps += [dep_name]

print_item("name",instance._name)
print_item("scope",instance.scopestr)
print_item("manifest present",instance.exists)
print_item("source present",src_present)
print_item("binaries present",bin_present)
print_item("architectures",instance._archlist)
print_item("declared deps",decl_deps)

print_item("source root",instance.source_root)
print_item("build_src",instance.build_src)
print_item("build_dest",instance.build_dest)




###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, multiprocessing 
from obt import target, host

class OsDescriptor:
  def __init__(self):
    self.id = ""
    self.version_id = ""
    self.version_codename = ""

def descriptor():
  try:
    import os_release
    INFO = os_release.current_release()
    osd = OsDescriptor()
    osd.id = INFO.id
    osd.version_id = INFO.version_id
    osd.version_codename = INFO.version_codename
    return osd
  except ModuleNotFoundError:
    try:
      import lsb_release
      INFO = lsb_release.get_os_release()
      osd = OsDescriptor()
      osd.id = INFO["ID"]
      osd.version_id = INFO["RELEASE"]
      osd.version_codename = INFO["CODENAME"]
      return osd
    except ModuleNotFoundError:
      print("python module 'lsb_release' or 'os_release' is not installed")
      return {}
    
  print("UHOH")
  return {}
  
  


###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, os, pathlib,sys
import obt.module

###############################################################################

file_path = os.path.realpath(__file__)
this_dir = pathlib.Path(os.path.dirname(file_path))

###############################################################################

def descriptor(architecture,osname):
  import obt.module
  import obt.path
  identifier = "%s-%s" % (architecture,osname)
  hi_name = obt.path.modules()/"sdk"
  hi_name = hi_name / ("%s.py"%identifier)
  the_module = obt.module.instance(identifier,hi_name)
  if the_module != None:
    hostident = obt.host.description().identifier
    if hasattr(the_module,"sdkinfo"):
      sdkinfo = the_module.sdkinfo()
      if hostident in sdkinfo.supports_host:
        return sdkinfo
  return None

###############################################################################

class enuminterface:
  def __init__(self):
    self.subdir = "sdk"
  def tryAsModule(self,hostidentifier,item,pth):
    identifier = item.replace(".py","")
    m = obt.module.instance(identifier,pth)
    if hasattr(m,"sdkinfo"):
      sdki = m.sdkinfo()
      if hostidentifier in sdki.supports_host:
        return m
    else:
      return None

###############################################################################

def enumerate():
  iface = enuminterface()
  return obt.module.enumerate_simple(iface)



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################


import os, sys, string
import shutil, glob
import platform
import subprocess
import os
import errno
import time
import sys
import xml.etree.ElementTree as xml
import hashlib

import obt.common
from obt.command import Command

deco = obt.deco.Deco()

def is_irix():
	return sys.platform.find("irix")!=-1

def pexec(args):
    return subprocess.Popen(string.split(args), stdout=subprocess.PIPE, shell=True).communicate()[0]

num_cores = 1

if is_irix():
	num_cores_str = pexec( 'hinv -c processor' )
	f = string.split(num_cores_str)[0]
	num_cores = int(float(f))
	print("IRIX num_cores<%d>" % num_cores)
else:
   try:
     import multiprocessing
     num_cores = multiprocessing.cpu_count()
   except ImportError:
     print("No MultiProcessing module - building with 1 core")
  


from obt.build.manifest import *
from obt.pathtools import *

import select
import fcntl

stage_dir = os.environ["OBT_STAGE"]

###############################################################################

#MARK: yo

SYSTEM = platform.system()

def IsCygwin():
	return SYSTEM.lower().find( "cygwin" )!=-1

def IsWindows():
	iswin = SYSTEM.lower().find( "windows" )!=-1
	ismso = SYSTEM.lower().find( "microsoft" )!=-1
	return iswin or ismso

###########################################

def folder_tree(rootpath):
	ret = list()
	ret.append(rootpath)
	#print "folder_tree<%s>"%rootpath
	for root, subFolders, files in os.walk(rootpath):
		for folder in subFolders:
			f = os.path.join(root,folder)
			ret.append(f)
			#print(f)
	return ret

###########################################
# check for obt.build projects
###########################################

def check_for_project(path):
	rval = None
	prj_manifest = "%s/obt.build.manifest"%path
	prj_scripts = os.path.abspath("%s/scripts"%path)
	#print "checking for project at<%s>" % deco.path(path)
	if os.path.exists(prj_manifest):
		#print "/////////////////////////////////////////"
		#print "// Projects Found !! <%s>" % deco.path(path)
		#print "/////////////////////////////////////////"
		###############
		prj = manifests.add_project(path)
		manifest_tree = xml.parse(prj_manifest)
		###############
		#print manifest_tree
		rootElement = manifest_tree.getroot()
		prj.manifest_root = rootElement
		for a in rootElement.findall('depends_on'):
			#print dir(a)
			depends = a.attrib["project"]
			#print "project<%s> depends<%s>" % (path,depends)

			manifests.depends(path,depends)
		###############
		prj.autoexecs = list()
		if os.path.exists(prj_scripts):
			sys.path.append(prj_scripts)
			append_env("PATH",prj_scripts)
			prj.scripts_folder = prj_scripts
			###############
			for a in rootElement.findall('autoexec'):
				scr = a.attrib["script"]
				scrpath = "%s/setenv.py" % prj_scripts
				if os.path.exists(scrpath):
					prj.autoexecs.append(scrpath)
		###############
		#prj.libdir = os.path.abspath("%s/lib " % path)
		rval = prj
	return rval
###########################################
def check_for_projects(base):
	manifests.add_project("%s/obt.build"%base)
	paths = glob.glob("%s/*"%base)
	PRJ_LIBDIRS=" "
	setenv_scrs = list()
	for p in paths:
		if p!="scripts":
			if os.path.isdir(p):
				prj = check_for_project(p)
				if prj!=None:
					if hasattr(prj,"libdir"):
						PRJ_LIBDIRS += prj.libdir
					if hasattr(prj,"autoexecs"):
						for a in prj.autoexecs:
							#print "execute autoexec<%s>" % a
							setenv_scrs.append(a)
	#append_env("PRJ_LIBDIRS",PRJ_LIBDIRS)
	for i in setenv_scrs:
		#print "exec child setenv <%s>" % i
		execfile(i)
	#print "PRJ_LIBDIRS<%s>" % PRJ_LIBDIRS
############################
def untar(arc,strip=False):
	is_gz = arc.find(".gz")>0
	is_bz2 = arc.find(".bz2")>0
	cmd = ["tar","--extract","--verbose"]
	if is_gz:
		cmd += ["--ungzip"]
	elif is_bz2:
		cmd += ["--bzip2"]
	if strip:
		cmd += ["--strip-components=1"]
	cmd += ["-f",arc]
	Command(cmd).exec()
#############################################
class context:
	############################
	def __init__(self,dl_dir):
		self.dl_dir = dl_dir
		self.opt_apts = False
		self.opt_exts = False
		self.opt_force = False

		self.nargs = len(sys.argv)
		if 1 == self.nargs:
			print("usage: (all | apts | exts | assets) [force]")
		else:
			for i in range(1,self.nargs,1):
				arg = sys.argv[i]
				if arg=="all":
					self.opt_apts = True
					self.opt_exts = True
				elif arg=="apts":
					self.opt_apts = True
				elif arg=="exts":
					self.opt_exts = True
				elif arg=="force":
					self.opt_force = True
		#print "opt_force<%s>" % self.opt_force
	############################
	def md5_of_file(self, fname):
		fil = open(fname,"rb")
		data = fil.read()
		fil.close()
		md5obj = hashlib.md5()
		md5obj.update(data)
		return md5obj.hexdigest()
	############################
	def wget(self,url,outputname,desmd5):
		out_path = "%s/%s" % (self.dl_dir,outputname)
		actmd5 = False
		if os.path.exists(out_path):
			actmd5 = self.md5_of_file(out_path)
		if actmd5!=desmd5:  
			cmd = "wget %s -O %s" % (url,out_path)
			os.system(cmd)
			actmd5 = self.md5_of_file(out_path)
		print("<%s> DesiredMD5<%s>" % (outputname,desmd5))
		print("<%s> ActualMD5<%s>" % (outputname,actmd5))
	############################
	def gitget( self, url, dir, rev=None ):
		os.chdir(self.dl_dir)
		if self.opt_force:
			assert(False)
			os.system( "rm -rf %s" % dir )
		exists = os.path.exists(dir)
		if exists:
			os.chdir(dir)
			os.system("git pull origin master")
		else:
			os.system("git clone %s %s" % (url,dir))
			os.chdir(dir)
		if rev!=None:
			os.system( "git checkout %s" % rev )
	############################
	def svnget( self, url, dir, rev="HEAD" ):
		os.chdir(self.dl_dir)
		if self.opt_force:
			os.system( "rm -rf %s" % dir )
		exists = os.path.exists(dir)
		if False==exists:
			os.system("svn checkout %s %s" % (url,dir))
		os.chdir(dir)
		os.system("svn up -r %s" % rev )
	############################
	def hgget( self, url, dir, rev="" ):
		os.chdir(self.dl_dir)
		if self.opt_force:
			os.system( "rm -rf %s" % dir )
		exists = os.path.exists(dir)
		if False==exists:
			os.system( "hg clone %s %s" % (url,dir) )
		os.chdir(dir) 	
		os.system("hg update %s" % rev)
	############################



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, host, command, path

###############################################################################

class ffmpeg(dep.StdProvider):
  name = "ffmpeg"
  def __init__(self):
    super().__init__(ffmpeg.name)
    #self._deps = ["pkgconfig"]
    src_root = self.source_root
    #################################################
    tgt_desc = self._target
    self._builder = self.createBuilder(dep.AutoConfBuilder)
    self._builder.setOption("--disable-vaapi")
    self._builder.setOption("--disable-vdpau")
    self._builder.setOption("--disable-static")
    self._builder.setOption("--enable-shared")
    if host.IsDarwin:
      self._builder.setEnvVar("LDFLAGS", '-Wl,-ld_classic')
      self._builder.setOption("--enable-videotoolbox")
    #elif host.IsLinux and host.IsX86_64:
    #  self._builder.setOption("--enable-nvenc")
    if tgt_desc.identifier == "x86_64-macos":
      self._builder.setOption("--disable-x86asm")

    #################################################
    self.declareDep("pkgconfig")
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=ffmpeg.name,
                             repospec="FFmpeg/FFmpeg",
                             revision="n5.1.3",
                             recursive=False)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"configure").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libffmpeg.so").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "v5.2.1"

import os, tarfile
from obt import dep, host, path, git, pathtools, command, patch, env
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command
import obt.host
from obt import log

deco = Deco()

###############################################################################

class lua(dep.Provider):

  def __init__(self): ############################################
    super().__init__("lua")
    #print(options)
    self.source_root = path.builds()/"lua"
    self.build_dest = self.source_root
    self.header_dest = path.prefix()/"include"/"lua"
    self.manifest = path.manifests()/"lua"
    self.OK = self.manifest.exists()

  ########

  def __str__(self):
    return "lua (lua.org-source-%s)" % VERSION

  ########

  def env_init(self):
    log.marker("registering Lua SDK")

  ########

  def download_and_extract(self): #############################################

    command.system("rm -rf %s"%self.source_root)

    git.Clone("https://github.com/lua/lua",
              self.source_root,
              rev=VERSION,
              cache=False)

    patch_items = dict()
    patch_items["CC= clang-3.8"]="CC= g++"
    patch_items["CFLAGS= -Wall -O2 $(MYCFLAGS)"]="CFLAGS= -Wall -O2 $(MYCFLAGS) -fPIC"
    if obt.host.IsLinux:
        patch_items["MYLDFLAGS= $(LOCAL) -Wl,-E"]="MYLDFLAGS= $(LOCAL) -Wl,-E -fPIC"
    else:
        patch_items["MYLDFLAGS= $(LOCAL) -Wl,-E"]="MYLDFLAGS= $(LOCAL) -fPIC"
        patch_items["-lhistory"]=""

    patch.patch_with_dict(self.source_root/"makefile",patch_items)

  def build(self): ############################################################

    self.download_and_extract()
    os.chdir(str(self.source_root))

    cmd = ["make","-j",host.NumCores]

    if obt.host.IsOsx:
        cmd += ["MACOSX_DEPLOYMENT_TARGET=10.14"]

    self.ok = (0 == Command(cmd).exec())

    return self.install()

  def install(self):
    if self.ok:
      cmd = ["cp",self.source_root/"lua",path.bin()/"lua"]
      self.ok = (0 == Command(cmd).exec())
      if self.ok:
        cmd = ["cp",self.source_root/"liblua.a",path.prefix()/"lib"/"liblua.a"]
        self.ok = (0 == Command(cmd).exec())
        if self.ok:
          pathtools.mkdir(self.header_dest,clean=True)
          cmd = ["cp", self.source_root/"*.h",str(self.header_dest)+"/"]
          self.ok = (0 == command.system(cmd))
    return self.ok 

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"makefile").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.bin()/"lua").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "03edcafdda550e55e29bf48a682097028ae01306"
HASH = "1453e79e663ea3bd8b528ef20a17109b"

import os, tarfile
from obt import dep, host, path, git, cmake
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################

class minetest(dep.Provider):

  def __init__(self): ############################################
    super().__init__("minetest")
    #print(options)
    build_dest = path.builds()/"minetest"
    self.build_dest = build_dest
    self.manifest = path.manifests()/"minetest"
    self.OK = self.manifest.exists()
    self.fname = "minetest-%s.zip" % VERSION
    self._archlist = ["x86_64"]

  ########

  def __str__(self):
    return "Minetest (github-commit-%s-source)" % VERSION

  ########

  def download_and_extract(self): #############################################
    from yarl import URL
    url = URL("https://github.com/minetest/minetest/archive/%s.zip"%VERSION)

    self.arcpath = dep.downloadAndExtract([url],
                                          self.fname,
                                          "zip",
                                          HASH,
                                          self.build_dest)


    source_dir = self.build_dest/("minetest-%s"%VERSION)
    os.chdir(str(source_dir/"games"))
    git.Clone("https://github.com/minetest/minetest_game","minetest_game")

    os.chdir(str(source_dir/"games"/"minetest_game"/"mods"))

    # install mods

    git.Clone("https://github.com/sofar/luscious","luscious")
    git.Clone("https://notabug.org/TenPlus1/mobs_redo","mobs_redo")
    git.Clone("https://notabug.org/TenPlus1/mobs_animal","mobs_animal")
    git.Clone("https://notabug.org/TenPlus1/mobs_monster","mobs_monster")
    git.Clone("https://notabug.org/TenPlus1/mobs_npc","mobs_npc")
    git.Clone("https://notabug.org/TenPlus1/mobs_horse","mobs_horse")
    git.Clone("https://github.com/blert2112/mobs_sky.git","mobs_sky")
    git.Clone("https://github.com/FreeLikeGNU/goblins","goblins")
    git.Clone("https://github.com/maikerumine/mobs_mc","mobs_mc")

    git.Clone("https://github.com/tweakoz/minetest_tozcmd","tozcmd")

    for item in "technic mesecons pipeworks moreores digtron lightning".split():
      git.Clone("https://github.com/minetest-mods/%s"%item,item)

  def build(self): ############################################################

    psql = dep.require("postgresql").instance
    irrl = dep.require("irrlicht").instance
    luaj = dep.require("luajit").instance

    source_dir = self.build_dest/("minetest-%s"%VERSION)
    build_temp = source_dir/".build"
    print(build_temp)
    if build_temp.exists():
      Command(["rm","-rf",build_temp]).exec()

    build_temp.mkdir(parents=True,exist_ok=True)
    os.chdir(str(build_temp))
    cmakeEnv = {
    }
    if host.IsOsx:
      cmakeEnv["CMAKE_MACOSX_RPATH"]=1
      cmakeEnv["CMAKE_INSTALL_RPATH"]=path.prefix()/"lib"
      cmakeEnv["GETTEXT_INCLUDE_DIR"]="/usr/local/opt/gettext/include"
      cmakeEnv["GETTEXT_LIBRARY"]="/usr/local/opt/gettext/lib"

    cmake.context(root=source_dir,env=cmakeEnv).exec()
    return 0==Command(["make","-j",host.NumCores,"install"]).exec()

  def provide(self): ##########################################################
    if False==self.OK:
      self.download_and_extract()
      self.OK = self.build()
      if self.OK:
        self.manifest.touch()

    return self.OK




###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION_MAJOR = "3"
VERSION_MINOR = "11"
VERSION_MICRO = "6"
VERSION = "%s.%s.%s" % (VERSION_MAJOR,VERSION_MINOR,VERSION_MICRO)
HASH = "ed23dadb9f1b9fd2e4e7d78619685c79"

import os, tarfile, sys
from obt import dep, host, path, cmake, env, pip
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command
from obt import log

deco = Deco()


###############################################################################

class python_from_source(dep.Provider):

  def __init__(self,target=None): ############################################
    super().__init__("python")
    self._debug_build = False
    ##########################################
    if target==None:
      # target implied is host => INIT scope
      self.scope = dep.ProviderScope.INIT
    else:
      assert(False) # not supported yet
    ##########################################
    # TODO - remove recursion...
    #self.declareDep("pkgconfig")
    ##########################################
    #print(options)
    build_dest = path.builds()/"python"
    self.build_dest = build_dest
    self.manifest = path.manifests()/"python"
    self.OK = self.manifest.exists()
    self.fname = "Python-%s.tgz"%VERSION
    self.source_dir = self.build_dest/("Python-%s"%VERSION)

  ########  

  def __str__(self):
    return "Python3 (%s-source)" % VERSION

  ########

  def env_init(self):
    log.marker("registering Python(%s) SDK"%VERSION)
    env.set("OBT_PYTHON_SUBSPACE_BUILD_DIR",path.builds())
    env.set("OBT_PYLIB",self.library_dir)
    env.set("OBT_PYPKG",self.site_packages_dir)
    env.set("OBT_PYTHON_HEADER_PATH",self.include_dir)
    env.set("OBT_PYTHON_LIB_PATH",self.home_dir/"lib")
    env.set("OBT_PYTHON_PYLIB_PATH",self.pylib_dir)
    env.set("OBT_PYTHON_LIB_FILE",self.library_file)
    env.set("OBT_PYTHON_LIB_NAME",self.library_name)
    env.set("OBT_PYTHON_DECO_NAME",self._deconame)
    env.set("OBT_PYTHON_DECOD_NAME",self._deconame_d)
    env.set("OBT_PYTHONHOME",self.virtualenv_dir)
    if True: # WIP
      env.prepend("PATH",self.virtualenv_dir/"bin" )
    #env.set("VIRTUAL_ENV",self.virtualenv_dir)
    env.prepend("LD_LIBRARY_PATH",self.home_dir/"lib")
    env.prepend("PKG_CONFIG_PATH",self.library_dir/"pkgconfig")
    
  ########

  def env_goto(self):
    return {
      "pylib": str(self.library_dir),
      "pypkg": str(self.site_packages_dir)
    }

  ########

  def env_properties(self):
    return {
      "pylib": self.library_dir,
      "pypkg": self.site_packages_dir
    }

  ########

  @property
  def version(self):
    va = VERSION_MAJOR
    vb = VERSION_MINOR
    vc = VERSION_MICRO
    return "%s.%s.%s" % (va,vb,vc)
  ########
  @property
  def version_major(self):
    va = VERSION_MAJOR
    vb = VERSION_MINOR
    return "%s.%s" % (va,vb)
  ########
  @property
  def _deconame(self):
    return "python%s"%(self.version_major)
  ########
  @property
  def _deconame_d(self):
    a = self._deconame
    if self._debug_build:
      a+= "d"
    return a
  ########
  @property
  def library_dir(self):
    # todo - use pkgconfig ?
    return self.home_dir/"lib"
  ########
  @property
  def venvlibrary_dir(self):
    # todo - use pkgconfig ?
    return self.virtualenv_dir/"lib"
  ########
  @property
  def pylib_dir(self):
    # todo - use pkgconfig ?
    return self.library_dir/self._deconame
  ########
  @property
  def library_file(self):
    # todo - use pkgconfig ?
    return ("lib%s.%s"%(self._deconame_d,self.shlib_extension))
  ########
  @property
  def library_name(self):
    # todo - use pkgconfig ?
    return ("lib%s"%self._deconame_d)
  ########
  @property
  def site_packages_dir(self):
    # todo - use pkgconfig ?
    return self.venvlibrary_dir/self._deconame/"site-packages"
  ########
  @property
  def virtualenv_dir(self):
    return self.home_dir
  ########
  @property
  def home_dir(self):
    return path.Path(os.environ["OBT_PYTHONHOME"])
  ########
  @property
  def include_dir(self):
    return self.home_dir/"include"/self._deconame_d
  ########
  @property
  def executable(self):
    return self.virtualenv_dir/"bin"/"python3"
  ########

  def download_and_extract(self): #############################################

    url = "https://www.python.org/ftp/python/%s/%s"%(VERSION,self.fname)

    self.arcpath = dep.downloadAndExtract([url],
                                          self.fname,
                                          "gz",
                                          HASH,
                                          self.build_dest)


  def build(self): ############################################################
    pkgconfig = dep.require("pkgconfig")
    if pkgconfig == None:
      return False
      
    self.download_and_extract()
    build_temp = self.source_dir/".build"
    print(build_temp)
    if build_temp.exists():
      Command(["rm","-rf",build_temp]).exec()

    build_temp.mkdir(parents=True,exist_ok=True)
    os.chdir(str(build_temp))
    options = [
        "--prefix",self.home_dir,
        "--with-system-ffi",
        "--enable-shared",
        #"--enable-loadable-sqlite-extensions",
        "--with-ensurepip=install" # atomically build pip
    ]

    if self._debug_build:
      options += ["--with-pydebug"]

    env.set("CCFLAGS","-march=%s"%self._target.architecture)

    if host.IsOsx:
       from obt import macos, macos_homebrew
       sdkdir = path.osx_sdkdir()
       print(sdkdir)
       #options += ["--enable-universalsdk=%s"%sdkdir]

       sslpath = macos_homebrew.prefix_for_package("openssl@1.1")
       print(sslpath)
 
       xzpath = macos_homebrew.prefix_for_package("xz")
       print(xzpath)
 
       options += ["--with-openssl=%s"%sslpath]
        #export LDFLAGS="-L$(brew --prefix xz)/lib $LDFLAGS";  export CPPFLAGS="-I$(brew --prefix xz)/include $CPPFLAGS";  export PKG_CONFIG_PATH="$(brew --prefix xz)/lib/pkgconfig:$PKG_CONFIG_PATH"

    else:
       options += ["--with-openssl=/usr"]

    Command(["../configure"]+options).exec()
    OK = (0==Command(["make",
                      "-j", host.NumCores,
                      "install"]).exec())
    ################################
    # install default packages
    ################################
    if OK:
      env.prepend("LD_LIBRARY_PATH",self.home_dir/"lib")
      os.chdir(str(build_temp))
      obt_python = self.home_dir/"bin"/"python3"
      venv_python = self.virtualenv_dir/"bin"/"python3"
      Command([obt_python,"-m","pip","install","--upgrade","pip"]).exec()
      Command([obt_python,"-m","pip","install","virtualenv"]).exec()
      Command([obt_python,"-m","venv", self.virtualenv_dir]).exec()
      Command([venv_python,"-m","pip","install","--upgrade","pip"]).exec()

      modules =  ["yarl","toposort","pytest","os_release","pyyaml"]

      Command([venv_python,"-m","pip","install"]+modules).exec()

    ################################
    return OK

  def areRequiredSourceFilesPresent(self):
    return (self.source_dir/"LICENSE").exists()

  def areRequiredBinaryFilesPresent(self):
    return (self.executable).exists()


python = python_from_source




#!/usr/bin/env python3
import os, sys, subprocess, shlex, argparse, json, threading, select, time, logging
from yarl import URL
from pathlib import Path

parser = argparse.ArgumentParser(description='jinja-build')
parser.add_argument('--sha', metavar="sha", required=True )
parser.add_argument('--outputdir', metavar="outputdir", required=True )
parser.add_argument('--command', metavar="command", required=True, action='append' )
parser.add_argument('--workingdir', metavar="workingdir", required=True )
parser.add_argument("--verbose",action="store_true")

args = vars(parser.parse_args())

SCRIPTDIR=Path(os.path.dirname(os.path.realpath(__file__)))

################################################################################

OUTPUTDIR = Path(args["outputdir"]).absolute()
COMMANDS = args["command"]
WORKINGDIR = args["workingdir"]
REPOBASEDIR = OUTPUTDIR/"repo"
VERBOSE=args["verbose"]
SHA=args["sha"]

#########################################

BASEDIR=OUTPUTDIR/"builds"/SHA
REPODIR=OUTPUTDIR/"repo"
STAGE=BASEDIR/".stage"
LOGTEXTPATH=BASEDIR/"stdout.log"

print("SCRIPTDIR=%s"%SCRIPTDIR)
print("BASEDIR=%s"%BASEDIR)
print("SHA=%s"%SHA)
print("REPODIR=%s"%REPODIR)
print("STAGE=%s"%STAGE)
print("LOGTEXTPATH=%s"%LOGTEXTPATH)
print("COMMANDS<%s>"%COMMANDS)
#########################################
def dirsubst(inpstr):
  outstr = inpstr.replace("$STAGE",str(STAGE))
  outstr = outstr.replace("$REPODIR",str(REPODIR))
  return outstr
#########################################
os.system("mkdir -p %s"%BASEDIR)
os.system("rm -rf %s"%STAGE)
#########################################
WORKINGDIR = dirsubst(WORKINGDIR)

#########################################
# VERBOSE
#########################################

def VERBOSE_RUN(logfile,arg_list):
  logging.basicConfig(stream=sys.stdout,level=logging.INFO)
  logging.addLevelName(logging.INFO+2,'STDERR')
  logging.addLevelName(logging.INFO+1,'STDOUT')
  logger = logging.getLogger("JOB")
  print("VERBOSE run of %s"%cmdlist)
  os.chdir(WORKINGDIR)
  #######################################
  def logstream(stream,loggercb):
    loggercb("Running Job for Sha<%s>"%SHA)
    while True:
      out = stream.readline().decode("UTF-8")
      if out:
        loggercb(out.rstrip())
        try:
          logfile.write(out)
        except:
          pass
      else:
        break
  #######################################
  proc = subprocess.Popen(arg_list,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
  stdout_thread = threading.Thread(target=logstream,args=(proc.stdout,lambda s: logger.log(logging.INFO+1,s)))
  stderr_thread = threading.Thread(target=logstream,args=(proc.stderr,lambda s: logger.log(logging.INFO+2,s)))
  stdout_thread.start()
  stderr_thread.start()
  while stdout_thread.isAlive() and stderr_thread.isAlive():
    pass
  #######################################
  proc.wait()
  return proc.returncode 

#########################################
# QUIET
#########################################

def QUIET_RUN(logfile,arg_list):
  print("QUIET run of %s"%arg_list)
  class my_buffer(object):
    def __init__(self, fileobject, prefix):
      self._fileobject = fileobject
      self.prefix = prefix
    def fileno(self):
      return self._fileobject.fileno()
    def write(self, text):
      return self._fileobject.write('%s %s' % (self.prefix, text))
  my_out = my_buffer(logfile, '')
  my_err = my_buffer(logfile, '')
  os.chdir(WORKINGDIR)
  proc = subprocess.Popen(arg_list, stdout=my_out, stderr=my_err)
  proc.wait()
  return proc.returncode 

#########################################


with open(LOGTEXTPATH,"w") as logfile:
  for item in COMMANDS:
    COMMAND = dirsubst(item)
    #########################################
    print("Building Environment for sha<%s>"%SHA)
    arg_list = shlex.split(COMMAND)
    print(arg_list)
    os.environ["USER"] = "builder"
    executor = VERBOSE_RUN if VERBOSE else QUIET_RUN
    retcode = executor(logfile,arg_list)
    if retcode!=0:
      sys.exit(retcode)
  sys.exit(0)


#!/usr/bin/env python3 

import os, ci_common 

os.environ["DOCKER_BUILDKIT"]="1"

cmdlist = list()

secrets_dir = os.environ["SECRETSDIR"]

cmdlist += ["docker","build"]
cmdlist += ["-f", "master.dockerfile"]
cmdlist += ["-t", "obtcicd/master_focal"]
cmdlist += ["--secret", "id=ssh_public,src=%s/buildkey.rsa.pub"%secrets_dir]
cmdlist += ["--secret", "id=ssh_private,src=%s/buildkey.rsa"%secrets_dir]
cmdlist += ["."]

ci_common.sync_subprocess(cmdlist)




#!/usr/bin/env python3 

import os, ci_common 

os.environ["DOCKER_BUILDKIT"]="1"

cmdlist = list()

cmdlist += ["docker","build"]
cmdlist += ["-f", "worker-ub20.dockerfile"]
cmdlist += ["-t", "obtcicd/worker_focal"]
cmdlist += ["."]

ci_common.sync_subprocess(cmdlist)




###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "master"

import os, tarfile
from obt import dep, host, path, git, make, cmake
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################

class unittestpp(dep.Provider):

  def __init__(self): ############################################
    super().__init__("unittestpp")

    self.source_root = path.builds()/"unittestpp"
    self.build_dest = path.builds()/"unittestpp"/".build"
    self.manifest = path.manifests()/"unittestpp"

    self.OK = self.manifest.exists()

  ########

  def __str__(self):
    return "UnitTestPP (github-%s)" % VERSION

  ########

  def build(self): ############################################################

    git.Clone("https://github.com/tweakoz/unittestpp",self.source_root,VERSION)

    os.system("rm -rf %s"%self.build_dest)
    os.mkdir(self.build_dest)
    os.chdir(self.build_dest)
    cmake_ctx = cmake.context("..",env={
        "BUILD_SHARED_LIBS": "ON"
    })
    cmake_ctx.exec()
    return (make.exec("install")==0)

  def provide(self): ##########################################################

    if self.should_build:

      self.OK = self.build()
      if self.OK:
        self.manifest.touch()

    return self.OK



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, inspect, tarfile
from pathlib import Path
import importlib.util
import obt.path, obt.host
from obt.command import Command, run
from obt.deco import Deco
from obt.wget import wget
from obt import pathtools, cmake, make, path, git, host, _globals
from obt._dep_fetch import *
from obt._dep_build import *
from obt._dep_provider import *
from obt._dep_impl import *
from obt._dep_node import *
from obt._dep_dl import *
from obt._dep_x import *
from obt._dep_enumerate import *



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, host, git, command, path, env
from obt.deco import Deco
deco = Deco()

class qt5ct(dep.StdProvider):
  name = "qt5ct"
  def __init__(self): ############################################
    super().__init__(qt5ct.name)
    self._archlist = ["x86_64"]
    #################################################
    srcroot = path.Path(self.source_root)
    #################################################
    class Builder(dep.BaseBuilder):
      def __init__(self,name):
        super().__init__(qt5ct.name)
      def build(self,srcdir,blddir,incremental=False):
        dep.require(self._deps)
        return True
      def install(self,blddir):
        (srcroot/"qt5ct").chdir()
        print(srcroot)
        env = {"PREFIX":path.stage()}
        ok = command.run([ "qmake"],env)==0
        if ok:
          # TODO: we MUST fix this SUDO!
          print(deco.red("WARNING THIS WILL WANT SUDO"))
          ok = command.run([ "sudo","make", "install"])==0
        return ok
    #################################################
    self._builder = Builder(qt5ct.name)
  ########################################################################
  @property
  def _fetcher(self):
    fetcher = dep.SvnFetcher(qt5ct.name)
    fetcher._url = "svn://svn.code.sf.net/p/qt5ct/code"
    fetcher._revision = "trunk"
    return fetcher
  ########################################################################



#!/usr/bin/env python3

import os, distro

UBUNTU_VERSION = int(float(distro.version())*100.0)

os.system("sudo update-alternatives --install /usr/bin/python python /usr/bin/python3 1")

deplist = []

if UBUNTU_VERSION <= 2004:
  deplist =  ["gcc-8","g++-8","python-dev"] # not avail in ub22

deplist += ["libboost-dev","gcc-9","g++-9","gcc-10","g++-10","clang-12","clang","clang-format"]
deplist += ["g++-12"] # https://askubuntu.com/questions/1441844/todays-ubuntu-22-04-updates-seem-to-break-clang-compiler
deplist += ["libboost-filesystem-dev","libboost-system-dev","libboost-thread-dev"]
deplist += ["libboost-program-options-dev","libftdi-dev", "libfmt-dev"]
deplist += ["libglfw3-dev","libflac++-dev","scons","git"]
deplist += ["rapidjson-dev","graphviz","doxygen","libtiff-dev"]
deplist += ["portaudio19-dev", "pybind11-dev"]
deplist += ["libpng-dev"]
deplist += ["iverilog","nvidia-cg-dev","nvidia-cuda-dev", "nvidia-cuda-toolkit"]
deplist += ["libopenblas-dev"]
deplist += ["librtmidi-dev"]
deplist += ["texinfo","xmlto"]
deplist += ["libgtkmm-3.0-dev"]
deplist += ["libfltk1.3-dev","freeglut3-dev"]
deplist += ["libfontconfig1-dev"]
deplist += ["libfreetype6-dev"]
deplist += ["libx11-dev"]
deplist += ["libxext-dev"]
deplist += ["libxfixes-dev"]
deplist += ["libxi-dev"]
deplist += ["libxrender-dev"]
deplist += ["libx11-xcb-dev"]
deplist += ["libavformat-dev"]
deplist += ["libavcodec-dev"]
deplist += ["libswscale-dev"]
deplist += ["libssl-dev"]
deplist += ["wget","git","git-lfs", "vim","cmake","python3-pip", "nasm"]
deplist += ["m4","bison","flex"]
deplist += ["libcurl4-openssl-dev","libusb-1.0-0-dev", "libbz2-dev"]
deplist += ["libreadline-dev"]
deplist += ["libsqlite3-dev"]
deplist += ["libtbb-dev"]
deplist += ["openctm-tools"] # ctmviewer
deplist += ["openscad"] # for trimesh
deplist += ["libclang-dev"]
deplist += ["libgmp-dev","libmpfr-dev","texinfo","libmpc-dev"]
deplist += ["libx11-dev"]
deplist += ["libx11-xcb-dev"]
deplist += ["libxext-dev"]
deplist += ["libxfixes-dev"]
deplist += ["libxi-dev"]
deplist += ["libxrender-dev"]
deplist += ["libxcb1-dev"]
deplist += ["libxcb-glx0-dev"]
deplist += ["libxcb-keysyms1-dev"]
deplist += ["libxcb-image0-dev"]
deplist += ["libxcb-shm0-dev"]
deplist += ["libxcb-icccm4-dev"]
deplist += ["libxcb-sync-dev"]
deplist += ["libxcb-xfixes0-dev"]
deplist += ["libxcb-shape0-dev"]
deplist += ["libxcb-randr0-dev"]
deplist += ["libxcb-render-util0-dev"]
deplist += ["libxcb-xinerama0-dev"]
deplist += ["libxkbcommon-dev"]
deplist += ["libxkbcommon-x11-dev"]
deplist += ["libxcb-xkb-dev"]
deplist += ["libxcb-cursor-dev"]
deplist += ["libxcb-util-dev"]
deplist += ["libmad0-dev","libsdl2-dev","libassimp-dev"]
deplist += ["device-tree-compiler"]
deplist += ["imagemagick","curl","tk-dev"]
deplist += ["libgeos-dev","libpng-dev","libspatialindex-dev"]
deplist += ["qt5-style-plugins","qt5ct","python3-gdal","python3-pyqt5","python3-pyqt5.qtopengl"]
deplist += ["python3-simplejson","python3-tk"]

deplist += ["libdrm-dev","libaudiofile-dev","libsndfile1-dev"]
deplist += ["libglew-dev"]

merged = " ".join(deplist)
os.system("sudo apt -y install %s" % merged)

os.system("pip3 install os_release")



#!/usr/bin/env python3
##########################################

from obt import dep 
import sys

##########################################
depnodes = dep.DepNode.ALL()
word = sys.argv[-2]
##########################################
for key in depnodes:
    if key.startswith(word):
        val = depnodes[key]
        if hasattr(val.instance,"on_build_shell") and val.instance.supports_host:
            print(key)
